#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <avr/pgmspace.h>
#include "ip_arp_udp_tcp.h"
#include "enc28j60.h"
#include "timeout.h"
#include "avr_compat.h"
#include "net.h"

//SERIAL:
int uart_putchar(char c, FILE *unused);
int uart_getchar(FILE* f);
FILE mystdout = FDEV_SETUP_STREAM(uart_putchar, NULL, _FDEV_SETUP_WRITE);
FILE mystdin =  FDEV_SETUP_STREAM(NULL, uart_getchar, _FDEV_SETUP_READ);


// Please modify the following two lines.
// Attention! MAC and IP have to be unique in your local area network.
// You can not have the same numbers in two devices. If a conflict appears
// change it!
static uint8_t mymac[6]     =   {0x00, 0x17, 0x31, 0x49, 0x39, 0x8F};
static uint8_t myip[4]      =   {169, 254, 11, 108};
#define MY_IP_STR               "169.254.11.108"

// Your IP.
uint8_t yip[4];

// Base url (you can put a DNS name instead of an IP address if you have
// a DNS server (baseurl must end in "/").
//static char baseurl[]       =   "http://169.254.11.108/";

// Listen port for tcp/www (max range 1-254).
static uint16_t mywwwport   =   80;

#define BUFFER_SIZE             800

unsigned char buf[BUFFER_SIZE + 1];
uint8_t page = 1;
uint8_t num = 0;
uint8_t temp=0;
uint8_t secunde = 0, minute = 0, ore = 0;
char temp_str[10];

int8_t analyse_get_url(char *str)
{
        uint8_t i = 0;

        while (*str && i < 10 && *str > ',' && *str < '{') {
                if (*str == '/') {
                        page = 1;
                        ++str;
                        break;
                } else if (*str == '?') {
                        num = 0;
                        page = 1;
                        ++str;
                        break;
                }
                ++i;
                ++str;
        }

        while (*str) {
                if (*str == 'c') {
                        ++str;
                        page = 0;
                } else if (*str == 'f' || *str == 's') {
                        page = 0;
                        break;
                } else if (*str == 't') {
                        page = 2;
                        break;
                } else if (*str == 'l') {
                        page = 3;
                        break;
                } else if (*str == 'b') {
                        page = 4;
                        break;
                } else if (*str == 'h') {
                        page = 5;
                        break;
                } else if (*str == 'd') {
                        page = 6;
                        break;
                } else {
                        break;
                }

                num |= (1 << ((*str-0x30)-1));
                str += 4;
        }

        return num;
}

uint16_t print_webpage(uint8_t *buf, uint8_t on_off)
{
        char buffer[30];
        uint16_t plen;
	sprintf(buffer,"<HTML>%u</HTML>\n",temp);
        plen = fill_tcp_data_p(buf, 0, PSTR("HTTP/1.0 200 OK\r\nContent-Type: text/html\r\n\r\n"));
	plen = fill_tcp_data_p(buf, plen, buffer);
	//printf("%s\n",buffer);
        return plen;
}

void initADC()
{
	ADMUX=(1<<REFS0);// For Aref=AVcc;
	ADCSRA=(1<<ADEN)|(7<<ADPS0);
}

uint16_t ReadADC(uint8_t ch)
{
   //Select ADC Channel ch must be 0-7
   ch=ch&0b00000111;
   ADMUX|=ch;

   //Start Single conversion

   ADCSRA|=(1<<ADSC);

   //Wait for conversion to complete
   while(!(ADCSRA & (1<<ADIF)));

   //Clear ADIF by writing one to it
   ADCSRA|=(1<<ADIF);

   return(ADC);
}


void initTimer(void)
{
// Initializare timer 1 compare match A pentru a avea un timer la nivel de 1 secunda, 16MHz
	OCR1A = 15624;
	TCCR1A = 0;
	TCCR1B = (1<<WGM12)|(1<<CS12)|(1<<CS10);
	TIMSK |= _BV(OCIE1A);
}
/************************************************************************ 
 * Vectorul de intreruperi... 
 ************************************************************************/
// Exemplu de functie ce se apeleaza la interval de 1 secunda, intrerupere generata de timerul 1 (vezi laboratorul 2) 
SIGNAL(SIG_OUTPUT_COMPARE1A)
{	
	// Cod ce se apeleaza 1 data pe secunda
	uint8_t val;
	val = ReadADC(0)/2; //citim valoarea de la ADC
	//printf("\nTemp = %u \n", val); //afisam valoarea citita
	temp=val;
	sprintf(temp_str,"%u",temp);
	
}

void USART_init(void)
{
	//TODO
	// Initializare modul USART pt comunicatie la 9600bps, 8 biti de date, la 16MHz											
	uint8_t ubrr;
	ubrr=103;
	UBRRL=(unsigned char)ubrr;
	UCSRB = (1<<RXEN) | (1<<TXEN);
	UCSRC = (1<<URSEL) | (1<<USBS) | (3<<UCSZ0);
}

int main(void) {
	//char buffer[30];
        uint16_t plen;
        uint16_t dat_p;
        uint8_t i = 0;
        int8_t cmd;
        delay_ms(10);
//SERIAL
	USART_init();
	
	stdin = &mystdin;
	stdout = &mystdout;
//SERIAL END
	initADC();
	printf("ADC init done\n");
        initTimer();
	printf("Time init done\n");
        sei();
        delay_ms(10);
        // Set output to Vcc, reset inactive.
	// Enable PD4, reset as output.
        DDRD |= (1<<DDD4);

        // Set output to gnd, reset the ethernet chip.
        PORTD &= ~(1<<PD4);
        delay_ms(10);

        // Set output to Vcc, reset inactive.
        PORTD |= (1<<PD4);
        delay_ms(200);

        // Initialize enc28j60.
        enc28j60Init(mymac);
	printf("enc28j60 init done\n");
	printf("My ip is MY_IP_STR");
        delay_ms(20);

        // Enable PB1, LED as output.
        DDRB |= (1<<DDB3);

        // Set output to Vcc, LED off.
        PORTB |= (1<<PB3);

        enc28j60PhyWrite(PHLCON, 0x476);
        delay_ms(20);

        // Set output to GND, red LED on.
        PORTB &= ~(1<<PB3);
        i = 1;

        // Init the ethernet/ip layer.
        init_ip_arp_udp_tcp(mymac, myip, mywwwport);

        while (1) {
                // Get the next new packet.
                plen = enc28j60PacketReceive(BUFFER_SIZE, buf);
                // plen will be different than zero if there is a valid
                // packet (without crc error).
                if (plen == 0) {
                        continue;
                }
		else{
			printf("Packet received\n");
		}
                // Arp is broadcast if unknown but a host may also
                // verify the mac address by sending it to
                // a unicast address.
                if (eth_type_is_arp_and_my_ip(buf, plen)) {
			printf("ARP received\n");
                        make_arp_answer_from_request(buf);
                        continue;
                }

                // Check if ip packets are for us.
                if (eth_type_is_ip_and_my_ip(buf,plen) == 0) {
                        continue;
                }

                if (buf[IP_PROTO_P] == IP_PROTO_ICMP_V && buf[ICMP_TYPE_P] == ICMP_TYPE_ECHOREQUEST_V) {
                        // A ping packet, let's send pong.
			printf("Ping received\n");
                        make_echo_reply_from_request(buf,plen);
                        continue;
                }

                // TCP port www start, compare only the lower byte.
                if (buf[IP_PROTO_P] == IP_PROTO_TCP_V && buf[TCP_DST_PORT_H_P] == 0 && buf[TCP_DST_PORT_L_P] == mywwwport) {
                        // Read source IP.
                        yip[0] = buf[26];
                        yip[1] = buf[27];
                        yip[2] = buf[28];
                        yip[3] = buf[29];

                        if (buf[TCP_FLAGS_P] & TCP_FLAGS_SYN_V) {
                                make_tcp_synack_from_syn(buf);
                                // make_tcp_synack_from_syn does already send the syn,ack.
                                continue;
                        }

                        if (buf[TCP_FLAGS_P] & TCP_FLAGS_ACK_V) {
                                init_len_info(buf);

                                // We can possibly have no data, just ack.
                                dat_p = get_tcp_data_pointer();
                                if (dat_p == 0) {
                                        if (buf[TCP_FLAGS_P] & TCP_FLAGS_FIN_V) {
                                                // Finack, answer with ack.
                                                make_tcp_ack_from_any(buf);
                                        }

                                        // Just an ack with no data, wait for next packet.
                                        continue;
                                }

                                if (strncmp("GET ", (char *)&(buf[dat_p]), 4) != 0) {
                                        // Head, post and other methods.
                                        // For possible status codes see:
                                        // http://www.w3.org/Protocols/rfc2616/rfc2616-sec10.html
                                        plen = fill_tcp_data_p(buf,0,PSTR("HTTP/1.0 200 OK\r\nContent-Type: text/html\r\n\r\n<h1>200 OK</h1>"));
                                        goto SENDTCP;
                                }
                                if (strncmp("/ ", (char *)&(buf[dat_p + 4]), 2) == 0) {
                                        plen=fill_tcp_data_p(buf, 0, PSTR("HTTP/1.0 200 OK\r\nContent-Type: text/html\r\n\r\n"));
                                        plen=fill_tcp_data_p(buf, plen, PSTR("<temperature>"));
                                        plen=fill_tcp_data(buf, plen, temp_str);
                                        plen=fill_tcp_data_p(buf, plen, PSTR("</temperature>"));
                                        goto SENDTCP;
                                }
                                cmd = analyse_get_url((char *)&(buf[dat_p + 5]));

                                if (page == 0) // aprins/stins ledurile
                                        PORTC = cmd;

                                plen = print_webpage(buf,cmd);
SENDTCP:
                                make_tcp_ack_from_any(buf);  // send ack for http get
                                make_tcp_ack_with_data(buf,plen);  // send data
                                continue;
                        }
                }
        }

        return 0;
}


/************************************************************************ 
 * IMPLEMENTARE FUNCTII TRANSMISIE/RECEPTIE SERIALA, pentru mapare stdin, stdout
 ************************************************************************/
// Functie ce trateaza trimiterea unui caracter pe seriala
int uart_putchar(char c, FILE *unused)      
{
	if (c == '\n')
		uart_putchar('\r', 0);
	loop_until_bit_is_set(UCSRA, UDRE);
	UDR = c;
	return 0;
}

// Functie ce trateaza primirea unui caracter pe seriala
int uart_getchar(FILE* f)
{
	char c;
    
    loop_until_bit_is_set(UCSRA, RXC);
    c = UDR;
    if(c=='\r')
		c='\n';
    uart_putchar(c,NULL); 
    return c;
}

